﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Collections;
using HeuristicLab.Data;
using HeuristicLab.Optimization;

namespace HeuristicLab.Problems.MetaOptimization.Encodings.Crossovers {
  /// <summary>
  /// 
  /// </summary>
  [Item("ParameterSetCrossover", "TODO")]
  [StorableClass]
  public class ParameterSetCrossover : SingleSuccessorOperator, IParameterSetOperator, ICrossover {
    public override bool CanChangeName {
      get { return false; }
    }

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<IParameterSet>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<IParameterSet>)Parameters["Parents"]; }
    }
    public ILookupParameter<IParameterSet> ChildParameter {
      get { return (ILookupParameter<IParameterSet>)Parameters["Child"]; }
    }

    public IValueParameter<IBinaryVectorCrossover> BinaryVectorCrossoverParameter {
      get { return (IValueParameter<IBinaryVectorCrossover>)Parameters["BinaryVectorCrossover"]; }
    }
    public IValueParameter<IIntegerVectorCrossover> IntegerVectorCrossoverParameter {
      get { return (IValueParameter<IIntegerVectorCrossover>)Parameters["IntegerVectorCrossover"]; }
    }
    public IValueParameter<IRealVectorCrossover> RealVectorCrossoverParameter {
      get { return (IValueParameter<IRealVectorCrossover>)Parameters["RealVectorCrossover"]; }
    }
    public ParameterSetCrossover()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic crossover operators."));
      Parameters.Add(new ScopeTreeLookupParameter<IParameterSet>("Parents", "The parent vectors which should be crossed."));
      ParentsParameter.ActualName = "ParameterSet";
      Parameters.Add(new LookupParameter<IParameterSet>("Child", "The child vector resulting from the crossover."));
      ChildParameter.ActualName = "ParameterSet";
      Parameters.Add(new ValueParameter<IRealVectorCrossover>("BinaryVectorCrossover", "Crossover operator used for crossing binary parameters"));
      Parameters.Add(new ValueParameter<IRealVectorCrossover>("IntegerVectorCrossover", "Crossover operator used for crossing integer parameters"));
      Parameters.Add(new ValueParameter<IRealVectorCrossover>("RealVectorCrossover", "Crossover operator used for crossing real parameters"));
    }
    public override IOperation Apply() {
      IEnumerable<IndexedItem<IParameterConfiguration>> integerParameters1 = GetIntegerParameters(ParentsParameter.ActualValue[0]);
      IEnumerable<IndexedItem<IParameterConfiguration>> integerParameters2 = GetIntegerParameters(ParentsParameter.ActualValue[1]);
      IParameterSet child = (IParameterSet)ParentsParameter.ActualValue[0].Clone();
      
      // TODO: use specific crossover operators to do the crossing
      // for now: just swap the int-values
      foreach (var par in integerParameters1) {
        ParentsParameter.ActualValue[1].Parameters.ElementAt(par.Index).Parameter.ActualValue = par.Value.Parameter.ActualValue;
      }
      foreach (var par in integerParameters2) {
        ParentsParameter.ActualValue[0].Parameters.ElementAt(par.Index).Parameter.ActualValue = par.Value.Parameter.ActualValue;
      }

      ChildParameter.ActualValue = child;
      return base.Apply();
    }

    private IEnumerable<IndexedItem<IParameterConfiguration>> GetIntegerParameters(IParameterSet parameterSet) {
      List<IndexedItem<IParameterConfiguration>> integerParameters = new List<IndexedItem<IParameterConfiguration>>();
      int i = 0;
      foreach (var item in parameterSet.Parameters) {
        if (item.Parameter.DataType == typeof(IntValue)) {
          integerParameters.Add(new IndexedItem<IParameterConfiguration>(i, (IParameterConfiguration)item.Clone()));
        }
        i++;
      }
      return integerParameters;
    }

  }
}
