﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class ParameterSet : Item, IParameterSet {
    [Storable]
    private List<IParameterConfiguration> parameters;
    public IEnumerable<IParameterConfiguration> Parameters {
      get { return parameters; }
    }

    public ParameterSet() {
      this.parameters = new List<IParameterConfiguration>();
    }

    [StorableConstructor]
    protected ParameterSet(bool deserializing) : base(deserializing) { }

    public ParameterSet(ParameterConfigurationList parametersToOptimize, IRandom random)
      : this() {
      foreach (IParameterConfiguration config in parametersToOptimize.CheckedItems) {
        IParameterConfiguration clone = (IParameterConfiguration)config.Clone();
        clone.SetParameterWithRandomValue(random);
        parameters.Add(clone);
      }
    }

    #region Cloning
    public override Common.IDeepCloneable Clone(Common.Cloner cloner) {
      ParameterSet clone = (ParameterSet)base.Clone(cloner);
      clone.parameters = new List<IParameterConfiguration>();
      foreach (var param in this.parameters) {
        clone.parameters.Add((IParameterConfiguration)param.Clone(cloner));
      }
      return clone;
    }
    #endregion
  }
}
